/*
    IA32
    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "w32osdll.h"
#include "aclui.h"
#include "activeds.h"
#include "advapi32.h"
#include "asycfilt.h"
#include "atl.h"
#include "authz.h"
#include "avrt.h"
#include "bcrypt.h"
#include "cabinet.h"
#include "clfsw32.h"
#include "clusapi.h"
#include "comctl32.h"
#include "comdlg32.h"
#include "comsvcs.h"
#include "credui.h"
#include "crtdll.h"
#include "crypt32.h"
#include "cryptnet.h"
#include "cryptui.h"
#include "ctl3d32.h"
#include "d3d10.h"
#include "d3d8thk.h"
#include "d3d9.h"
#include "dbgeng.h"
#include "dbghelp.h"
#include "dciman32.h"
#include "ddraw.h"
#include "dhcpcsvc.h"
#include "dhcpcsvc6.h"
#include "dhcpsapi.h"
#include "dinput.h"
#include "dnsapi.h"
#include "dplayx.h"
#include "dpx.h"
#include "dsound.h"
#include "dsprop.h"
#include "dsuiext.h"
#include "dwmapi.h"
#include "dxgi.h"
#include "dxva2.h"
#include "eappcfg.h"
#include "eappprxy.h"
#include "encapi.h"
#include "esent.h"
#include "evr.h"
#include "faultrep.h"
#include "framedyn.h"
#include "fwpuclnt.h"
#include "fxsutility.h"
#include "gdi32.h"
#include "glu32.h"
#include "gpedit.h"
#include "hlink.h"
#include "httpapi.h"
#include "icm32.h"
#include "icmui.h"
#include "imagehlp.h"
#include "imm32.h"
#include "infocardapi.h"
#include "iphlpapi.h"
#include "iprop.h"
#include "iscsidsc.h"
#include "kernel32.h"
#include "ksuser.h"
#include "ktmw32.h"
#include "loadperf.h"
#include "lz32.h"
#include "magnification.h"
#include "mapi32.h"
#include "mf.h"
#include "mfc100.h"
#include "mfc100u.h"
#include "mfc40.h"
#include "mfc40u.h"
#include "mfc42.h"
#include "mfc42d.h"
#include "mfc42u.h"
#include "mfc42ud.h"
#include "mfcm100.h"
#include "mfcm100u.h"
#include "mfcclwz.h"
#include "mfcd42d.h"
#include "mfcd42ud.h"
#include "mfcn42d.h"
#include "mfcn42ud.h"
#include "mfco42d.h"
#include "mfco42ud.h"
#include "mfplat.h"
#include "mgmtapi.h"
#include "mpr.h"
#include "mprapi.h"
#include "msacm32.h"
#include "mscms.h"
#include "mscoree.h"
#include "msctfmonitor.h"
#include "msdelta.h"
#include "msdmo.h"
#include "msdrm.h"
#include "msdtcprx.h"
#include "msdtctm.h"
#include "msi.h"
#include "msimg32.h"
#include "msrating.h"
#include "mstask.h"
#include "msvcirt.h"
#include "msvcirtd.h"
#include "msvcp100.h"
#include "msvcp100d.h"
#include "msvcp50.h"
#include "msvcp60.h"
#include "msvcp60d.h"
#include "msvcr100.h"
#include "msvcr100d.h"
#include "msvcr71.h"
#include "msvcr90.h"
#include "msvcrt.h"
#include "msvcrt20.h"
#include "msvcrt40.h"
#include "msvcrtd.h"
#include "mswsock.h"
#include "msxml6.h"
#include "mtxdm.h"
#include "mtxoci.h"
#include "ncrypt.h"
#include "nddeapi.h"
#include "ndfapi.h"
#include "ndproxystub.h"
#include "netapi32.h"
#include "newdev.h"
#include "normaliz.h"
#include "ntdll.h"
#include "ntdsapi.h"
#include "odbc32.h"
#include "odbcbcp.h"
#include "odbccp32.h"
#include "ole32.h"
#include "oleacc.h"
#include "oleaut32.h"
#include "oledlg.h"
#include "olepro32.h"
#include "opengl32.h"
#include "p2p.h"
#include "p2pgraph.h"
#include "pdh.h"
#include "powrprof.h"
#include "prntvpt.h"
#include "propsys.h"
#include "psapi.h"
#include "quartz.h"
#include "qutil.h"
#include "qwave.h"
#include "rasapi32.h"
#include "rasdlg.h"
#include "resutils.h"
#include "riched20.h"
#include "rpcns4.h"
#include "rpcrt4.h"
#include "rstrtmgr.h"
#include "rtm.h"
#include "rtutils.h"
#include "scarddlg.h"
#include "secur32.h"
#include "sensapi.h"
#include "setupapi.h"
#include "sfc.h"
#include "shell32.h"
#include "shfolder.h"
#include "shlwapi.h"
#include "sisbkup.h"
#include "slc.h"
#include "slcext.h"
#include "slwga.h"
#include "snmpapi.h"
#include "srclient.h"
#include "sti.h"
#include "structuredquery.h"
#include "tapi32.h"
#include "taskschd.h"
#include "tbs.h"
#include "tdh.h"
#include "traffic.h"
#include "uiautomationcore.h"
#include "ulib.h"
#include "url.h"
#include "urlmon.h"
#include "user32.h"
#include "userenv.h"
#include "usp10.h"
#include "uxtheme.h"
#include "vdmdbg.h"
#include "version.h"
#include "vssapi.h"
#include "wecapi.h"
#include "wer.h"
#include "wevtapi.h"
#include "win32spl.h"
#include "windowscodecs.h"
#include "winfax.h"
#include "winhttp.h"
#include "wininet.h"
#include "winmm.h"
#include "winsatapi.h"
#include "winscard.h"
#include "wintrust.h"
#include "wlanapi.h"
#include "wlanui.h"
#include "wldap32.h"
#include "wmdrmsdk.h"
#include "wmiutils.h"
#include "wmvcore.h"
#include "wow32.h"
#include "ws2_32.h"
#include "wscapi.h"
#include "wsdapi.h"
#include "wsnmp32.h"
#include "wsock32.h"
#include "wtsapi32.h"
#include "xmllite.h"
#include "xolehlp.h"


const Ia32Osdll ia32_osdll[] = {
    {0x74800000, "aclui.dll",            ia32_aclui_exports},
    {0x6e000000, "activeds.dll",         ia32_activeds_exports},
    {0x77c60000, "advapi32.dll",         ia32_advapi32_exports},
    {0x06220000, "asycfilt.dll",         ia32_asycfilt_exports},
    {0x6e400000, "atl.dll",              ia32_atl_exports},
    {0x6de00000, "authz.dll",            ia32_authz_exports},
    {0x75400000, "avrt.dll",             ia32_avrt_exports},
    {0x6d800000, "bcrypt.dll",           ia32_bcrypt_exports},
    {0x6c7e0000, "cabinet.dll",          ia32_cabinet_exports},
    {0x05e00000, "clfsw32.dll",          ia32_clfsw32_exports},
    {0x6cd00000, "clusapi.dll",          ia32_clusapi_exports},
    {0x6f770000, "comctl32.dll",         ia32_comctl32_exports},
    {0x6f050000, "comdlg32.dll",         ia32_comdlg32_exports},
    {0x77000000, "comsvcs.dll",          ia32_comsvcs_exports},
    {0x6ccc0000, "credui.dll",           ia32_credui_exports},
    {0x6c240000, "crtdll.dll",           ia32_crtdll_exports},
    {0x719b0000, "crypt32.dll",          ia32_crypt32_exports},
    {0x40f30000, "cryptnet.dll",         ia32_cryptnet_exports},
    {0x714d0000, "cryptui.dll",          ia32_cryptui_exports},
    {0x050e0000, "ctl3d32.dll",          ia32_ctl3d32_exports},
    {0x21c00000, "d3d10.dll",            ia32_d3d10_exports},
    {0x41000000, "d3d8thk.dll",          ia32_d3d8thk_exports},
    {0x75410000, "d3d9.dll",             ia32_d3d9_exports},
    {0x2d560000, "dbgeng.dll",           ia32_dbgeng_exports},
    {0x71110000, "dbghelp.dll",          ia32_dbghelp_exports},
    {0x40c00000, "dciman32.dll",         ia32_dciman32_exports},
    {0x71020000, "ddraw.dll",            ia32_ddraw_exports},
    {0x40ed0000, "dhcpcsvc.dll",         ia32_dhcpcsvc_exports},
    {0x40460000, "dhcpcsvc6.dll",        ia32_dhcpcsvc6_exports},
    {0x05000000, "dhcpsapi.dll",         ia32_dhcpsapi_exports},
    {0x419c0000, "dinput.dll",           ia32_dinput_exports},
    {0x6dc00000, "dnsapi.dll",           ia32_dnsapi_exports},
    {0x0c280000, "dplayx.dll",           ia32_dplayx_exports},
    {0x0e440000, "dpx.dll",              ia32_dpx_exports},
    {0x6ef80000, "dsound.dll",           ia32_dsound_exports},
    {0x09590000, "dsprop.dll",           ia32_dsprop_exports},
    {0x181f0000, "dsuiext.dll",          ia32_dsuiext_exports},
    {0x40e50000, "dwmapi.dll",           ia32_dwmapi_exports},
    {0x152d0000, "dxgi.dll",             ia32_dxgi_exports},
    {0x04de0000, "dxva2.dll",            ia32_dxva2_exports},
    {0x6cb80000, "eappcfg.dll",          ia32_eappcfg_exports},
    {0x40e10000, "eappprxy.dll",         ia32_eappprxy_exports},
    {0x10000000, "encapi.dll",           ia32_encapi_exports},
    {0x74450000, "esent.dll",            ia32_esent_exports},
    {0x13b80000, "evr.dll",              ia32_evr_exports},
    {0x107a0000, "faultrep.dll",         ia32_faultrep_exports},
    {0x6cac0000, "framedyn.dll",         ia32_framedyn_exports},
    {0x6ca80000, "fwpuclnt.dll",         ia32_fwpuclnt_exports},
    {0x0be80000, "fxsutility.dll",       ia32_fxsutility_exports},
    {0x77b60000, "gdi32.dll",            ia32_gdi32_exports},
    {0x08f10000, "glu32.dll",            ia32_glu32_exports},
    {0x32400000, "gpedit.dll",           ia32_gpedit_exports},
    {0x40d10000, "hlink.dll",            ia32_hlink_exports},
    {0x40390000, "httpapi.dll",          ia32_httpapi_exports},
    {0x0bcc0000, "icm32.dll",            ia32_icm32_exports},
    {0x5e980000, "icmui.dll",            ia32_icmui_exports},
    {0x41870000, "imagehlp.dll",         ia32_imagehlp_exports},
    {0x41840000, "imm32.dll",            ia32_imm32_exports},
    {0x50000000, "infocardapi.dll",      ia32_infocardapi_exports},
    {0x40c90000, "iphlpapi.dll",         ia32_iphlpapi_exports},
    {0x00400000, "iprop.dll",            ia32_iprop_exports},
    {0x03e40000, "iscsidsc.dll",         ia32_iscsidsc_exports},
    {0x77de0000, "kernel32.dll",         ia32_kernel32_exports},
    {0x40300000, "ksuser.dll",           ia32_ksuser_exports},
    {0x402f0000, "ktmw32.dll",           ia32_ktmw32_exports},
    {0x417d0000, "loadperf.dll",         ia32_loadperf_exports},
    {0x00400000, "lz32.dll",             ia32_lz32_exports},
    {0x10000000, "magnification.dll",    ia32_magnification_exports},
    {0x03aa0000, "mapi32.dll",           ia32_mapi32_exports},
    {0x324f0000, "mf.dll",               ia32_mf_exports},
    {0x78b60000, "mfc100.dll",           ia32_mfc100_exports},
    {0x785f0000, "mfc100u.dll",          ia32_mfc100u_exports},
    {0x4fdb0000, "mfc40.dll",            ia32_mfc40_exports},
    {0x4fcc0000, "mfc40u.dll",           ia32_mfc40u_exports},
    {0x71890000, "mfc42.dll",            ia32_mfc42_exports},
    {0x5f400000, "mfc42d.dll",           ia32_mfc42d_exports},
    {0x71d80000, "mfc42u.dll",           ia32_mfc42u_exports},
    {0x5f800000, "mfc42ud.dll",          ia32_mfc42ud_exports},
    {0x52a00000, "mfcclwz.dll",          ia32_mfcclwz_exports},
    {0x5f700000, "mfcd42d.dll",          ia32_mfcd42d_exports},
    {0x5f700000, "mfcd42ud.dll",         ia32_mfcd42ud_exports},
    {0x78a20000, "mfcm100.dll",          ia32_mfcm100_exports},
    {0x78a40000, "mfcm100u.dll",         ia32_mfcm100u_exports},
    {0x5f600000, "mfcn42d.dll",          ia32_mfcn42d_exports},
    {0x5f600000, "mfcn42ud.dll",         ia32_mfcn42ud_exports},
    {0x5f500000, "mfco42d.dll",          ia32_mfco42d_exports},
    {0x5f500000, "mfco42ud.dll",         ia32_mfco42ud_exports},
    {0x0fdc0000, "mfplat.dll",           ia32_mfplat_exports},
    {0x402a0000, "mgmtapi.dll",          ia32_mgmtapi_exports},
    {0x40bc0000, "mpr.dll",              ia32_mpr_exports},
    {0x41740000, "mprapi.dll",           ia32_mprapi_exports},
    {0x40ba0000, "msacm32.dll",          ia32_msacm32_exports},
    {0x6ee00000, "mscms.dll",            ia32_mscms_exports},
    {0x79000000, "mscoree.dll",          ia32_mscoree_exports},
    {0x40240000, "msctfmonitor.dll",     ia32_msctfmonitor_exports},
    {0x0d6c0000, "msdelta.dll",          ia32_msdelta_exports},
    {0x40230000, "msdmo.dll",            ia32_msdmo_exports},
    {0x0fca0000, "msdrm.dll",            ia32_msdrm_exports},
    {0x6fd70000, "msdtcprx.dll",         ia32_msdtcprx_exports},
    {0x2f400000, "msdtctm.dll",          ia32_msdtctm_exports},
    {0x759b0000, "msi.dll",              ia32_msi_exports},
    {0x40210000, "msimg32.dll",          ia32_msimg32_exports},
    {0x6c940000, "msrating.dll",         ia32_msrating_exports},
    {0x6c8c0000, "mstask.dll",           ia32_mstask_exports},
    {0x03520000, "msvcirt.dll",          ia32_msvcirt_exports},
    {0x102a0000, "msvcirtd.dll",         ia32_msvcirtd_exports},
    {0x78050000, "msvcp100.dll",         ia32_msvcp100_exports},
    {0x10480000, "msvcp100d.dll",        ia32_msvcp100d_exports},
    {0x780c0000, "msvcp50.dll",          ia32_msvcp50_exports},
    {0x6e6b0000, "msvcp60.dll",          ia32_msvcp60_exports},
    {0x10480000, "msvcp60d.dll",         ia32_msvcp60d_exports},
    {0x78aa0000, "msvcr100.dll",         ia32_msvcr100_exports},
    {0x10200000, "msvcr100d.dll",        ia32_msvcr100d_exports},
    {0x7c340000, "msvcr71.dll",          ia32_msvcr71_exports},
    {0x78520000, "msvcr90.dll",          ia32_msvcr90_exports},
    {0x6ff50000, "msvcrt.dll",           ia32_msvcrt_exports},
    {0x6d430000, "msvcrt20.dll",         ia32_msvcrt20_exports},
    {0x00400000, "msvcrt40.dll",         ia32_msvcrt40_exports},
    {0x10200000, "msvcrtd.dll",          ia32_msvcrtd_exports},
    {0x6c880000, "mswsock.dll",          ia32_mswsock_exports},
    {0x726a0000, "msxml6.dll",           ia32_msxml6_exports},
    {0x10000000, "mtxdm.dll",            ia32_mtxdm_exports},
    {0x415f0000, "mtxoci.dll",           ia32_mtxoci_exports},
    {0x6c800000, "ncrypt.dll",           ia32_ncrypt_exports},
    {0x40180000, "nddeapi.dll",          ia32_nddeapi_exports},
    {0x0b100000, "ndfapi.dll",           ia32_ndfapi_exports},
    {0x10000000, "ndproxystub.dll",      ia32_ndproxystub_exports},
    {0x40ac0000, "netapi32.dll",         ia32_netapi32_exports},
    {0x6e160000, "newdev.dll",           ia32_newdev_exports},
    {0x00400000, "normaliz.dll",         ia32_normaliz_exports},
    {0x77ec0000, "ntdll.dll",            ia32_ntdll_exports},
    {0x40a80000, "ntdsapi.dll",          ia32_ntdsapi_exports},
    {0x6f4a0000, "odbc32.dll",           ia32_odbc32_exports},
    {0x10000000, "odbcbcp.dll",          ia32_odbcbcp_exports},
    {0x03060000, "odbccp32.dll",         ia32_odbccp32_exports},
    {0x72540000, "ole32.dll",            ia32_ole32_exports},
    {0x6c490000, "oleacc.dll",           ia32_oleacc_exports},
    {0x6fc30000, "oleaut32.dll",         ia32_oleaut32_exports},
    {0x40a00000, "oledlg.dll",           ia32_oledlg_exports},
    {0x02fc0000, "olepro32.dll",         ia32_olepro32_exports},
    {0x19cc0000, "opengl32.dll",         ia32_opengl32_exports},
    {0x0af00000, "p2p.dll",              ia32_p2p_exports},
    {0x6e100000, "p2pgraph.dll",         ia32_p2pgraph_exports},
    {0x6c370000, "pdh.dll",              ia32_pdh_exports},
    {0x41470000, "powrprof.dll",         ia32_powrprof_exports},
    {0x41440000, "prntvpt.dll",          ia32_prntvpt_exports},
    {0x1d300000, "propsys.dll",          ia32_propsys_exports},
    {0x7daa0000, "psapi.dll",            ia32_psapi_exports},
    {0x76010000, "quartz.dll",           ia32_quartz_exports},
    {0x40940000, "qutil.dll",            ia32_qutil_exports},
    {0x0ad00000, "qwave.dll",            ia32_qwave_exports},
    {0x6dfa0000, "rasapi32.dll",         ia32_rasapi32_exports},
    {0x70730000, "rasdlg.dll",           ia32_rasdlg_exports},
    {0x40860000, "resutils.dll",         ia32_resutils_exports},
    {0x6ec80000, "riched20.dll",         ia32_riched20_exports},
    {0x5ea80000, "rpcns4.dll",           ia32_rpcns4_exports},
    {0x77bb0000, "rpcrt4.dll",           ia32_rpcrt4_exports},
    {0x41310000, "rstrtmgr.dll",         ia32_rstrtmgr_exports},
    {0x077a0000, "rtm.dll",              ia32_rtm_exports},
    {0x3ffc0000, "rtutils.dll",          ia32_rtutils_exports},
    {0x02c40000, "scarddlg.dll",         ia32_scarddlg_exports},
    {0x3ffa0000, "secur32.dll",          ia32_secur32_exports},
    {0x3ff90000, "sensapi.dll",          ia32_sensapi_exports},
    {0x734b0000, "setupapi.dll",         ia32_setupapi_exports},
    {0x00400000, "sfc.dll",              ia32_sfc_exports},
    {0x73800000, "shell32.dll",          ia32_shell32_exports},
    {0x3ff50000, "shfolder.dll",         ia32_shfolder_exports},
    {0x6de20000, "shlwapi.dll",          ia32_shlwapi_exports},
    {0x10000000, "sisbkup.dll",          ia32_sisbkup_exports},
    {0x3fef0000, "slc.dll",              ia32_slc_exports},
    {0x3fee0000, "slcext.dll",           ia32_slcext_exports},
    {0x10000000, "slwga.dll",            ia32_slwga_exports},
    {0x3ff20000, "snmpapi.dll",          ia32_snmpapi_exports},
    {0x3fed0000, "srclient.dll",         ia32_srclient_exports},
    {0x6c080000, "sti.dll",              ia32_sti_exports},
    {0x0f040000, "structuredquery.dll",  ia32_structuredquery_exports},
    {0x41c80000, "tapi32.dll",           ia32_tapi32_exports},
    {0x6ec00000, "taskschd.dll",         ia32_taskschd_exports},
    {0x10000000, "tbs.dll",              ia32_tbs_exports},
    {0x6f980000, "tdh.dll",              ia32_tdh_exports},
    {0x3fe80000, "traffic.dll",          ia32_traffic_exports},
    {0x14370000, "uiautomationcore.dll", ia32_uiautomationcore_exports},
    {0x406e0000, "ulib.dll",             ia32_ulib_exports},
    {0x41110000, "url.dll",              ia32_url_exports},
    {0x72400000, "urlmon.dll",           ia32_urlmon_exports},
    {0x77d10000, "user32.dll",           ia32_user32_exports},
    {0x406a0000, "userenv.dll",          ia32_userenv_exports},
    {0x6f8e0000, "usp10.dll",            ia32_usp10_exports},
    {0x6ce70000, "uxtheme.dll",          ia32_uxtheme_exports},
    {0x3fe10000, "vdmdbg.dll",           ia32_vdmdbg_exports},
    {0x3fe00000, "version.dll",          ia32_version_exports},
    {0x716e0000, "vssapi.dll",           ia32_vssapi_exports},
    {0x10000000, "wecapi.dll",           ia32_wecapi_exports},
    {0x6e480000, "wer.dll",              ia32_wer_exports},
    {0x6cdd0000, "wevtapi.dll",          ia32_wevtapi_exports},
    {0x6ea80000, "win32spl.dll",         ia32_win32spl_exports},
    {0x1d000000, "windowscodecs.dll",    ia32_windowscodecs_exports},
    {0x3fda0000, "winfax.dll",           ia32_winfax_exports},
    {0x0eae0000, "winhttp.dll",          ia32_winhttp_exports},
    {0x71200000, "wininet.dll",          ia32_wininet_exports},
    {0x41b00000, "winmm.dll",            ia32_winmm_exports},
    {0x0ea80000, "winsatapi.dll",        ia32_winsatapi_exports},
    {0x410a0000, "winscard.dll",         ia32_winscard_exports},
    {0x41010000, "wintrust.dll",         ia32_wintrust_exports},
    {0x405a0000, "wlanapi.dll",          ia32_wlanapi_exports},
    {0x10f90000, "wlanui.dll",           ia32_wlanui_exports},
    {0x6cd80000, "wldap32.dll",          ia32_wldap32_exports},
    {0x15580000, "wmdrmsdk.dll",         ia32_wmdrmsdk_exports},
    {0x40520000, "wmiutils.dll",         ia32_wmiutils_exports},
    {0x2bd90000, "wmvcore.dll",          ia32_wmvcore_exports},
    {0x0ffb0000, "wow32.dll",            ia32_wow32_exports},
    {0x41ac0000, "ws2_32.dll",           ia32_ws2_32_exports},
    {0x10000000, "wscapi.dll",           ia32_wscapi_exports},
    {0x6ea00000, "wsdapi.dll",           ia32_wsdapi_exports},
    {0x40480000, "wsnmp32.dll",          ia32_wsnmp32_exports},
    {0x3fd10000, "wsock32.dll",          ia32_wsock32_exports},
    {0x3fd00000, "wtsapi32.dll",         ia32_wtsapi32_exports},
    {0x09ec0000, "xmllite.dll",          ia32_xmllite_exports},
    {0x10000000, "xolehlp.dll",          ia32_xolehlp_exports},
    {0,          NULL,                   NULL}};
